package cn.iocoder.yudao.module.order.service.order;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderFeeApplication.OrderFeeApplicationDO;
import cn.iocoder.yudao.module.order.vo.orderFeeApplication.*;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 订单费用申请 Service 接口
 *
 * @author 系统管理员
 */
public interface OrderFeeApplicationService extends IService<OrderFeeApplicationDO> {

    /**
     * 创建订单费用申请
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createFeeApplication(@Valid OrderFeeApplicationCreateReqVO createReqVO) throws Exception;

    /**
     * 批量订单费用申请
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    void createFeeBatchApplication(@Valid OrderFeeBatchApplicationUpdateReqVO updateReqVO) throws Exception;

    /**
     * 更新订单费用申请
     *
     * @param updateReqVO 更新信息
     */
    void updateFeeApplication(@Valid OrderFeeApplicationUpdateReqVO updateReqVO);

    /**
     * 删除订单费用申请
     *
     * @param id 编号
     */
    void deleteFeeApplication(Long id);

    /**
     * 获得订单费用申请
     *
     * @param id 编号
     * @return 订单费用申请
     */
    OrderFeeApplicationDO getFeeApplication(Long id);

    /**
     * 获得订单费用申请列表
     *
     * @param ids 编号
     * @return 订单费用申请列表
     */
    List<OrderFeeApplicationDO> getFeeApplicationList(Collection<Long> ids);

    /**
     * 获得订单费用申请分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 订单费用申请分页
     */
    PageResult<OrderFeeApplicationDO> getFeeApplicationPage(OrderFeeApplicationQueryVO query, PageVO page);

    /**
     * 获得订单费用申请列表, 用于 Excel 导出
     *
     * @param query 查询
     * @return 订单费用申请列表
     */
    List<OrderFeeApplicationDO> getFeeApplicationList(OrderFeeApplicationQueryVO query);

    void updateFeeApproveResult(String id, int result);


    /**
     * 批量费用审核回调
     *
     * @param id
     * @param result
     */
    void updateFeeApproveBatchResult(String id, int result);

    List<OrderFeeApplicationDO> getFeeApplicationListByOrderId(Long orderId);

    OrderFeeApplicationBackVO getOrderByProcessId(String processId);

    List<OrderFeeApplicationBackVO> getBatchFeeByProcessId(String processId);

    void cancelFeeApplication(OrderFeeApplicationUpdateReqVO orderFeeApplicationUpdateReqVO);

    OrderFeeApplicationBackVO getApprovalFeeApplication(Long id);
}
